@@ -1,45 +1,13 @@
-# ===============================================================================
-#
-# PROGRAMMERS:
-#
-# jean-romain.roussel.1@ulaval.ca  -  https://github.com/Jean-Romain/lidR
-#
-# COPYRIGHT:
-#
-# Copyright 2016-2018 Jean-Romain Roussel
-#
-# This file is part of lidR R package.
-#
-# lidR is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>
-#
-# ===============================================================================
-
-
-
 #' LAScatalog processing engine
 #'
 #' This function gives users access to the \link[=LAScatalog-class]{LAScatalog} processing engine.
-#' It allows the application of a user-defined routine over an entire catalog. The LAScatalog
-#' processing engine tool is explained in the \link[=LAScatalog-class]{LAScatalog class}\cr\cr
+#' It allows the application of a user-defined routine over a collection of LAS/LAZ files. The
+#' LAScatalog processing engine is explained in the \link[=LAScatalog-class]{LAScatalog class}\cr\cr
 #' `catalog_apply()` is the core of the lidR package. It drives every single function that can process a
-#' \code{LAScatalog}. It is flexible and powerful but also complex. `catalog_sapply` is the same
-#' with the option `automerge = TRUE` enforced to simplify the output.\cr\cr
-#' **Warning:** the LAScatalog processing engine has a mechanism to load buffered data 'on-the-fly'
-#' to avoid edge artifacts, but no mechanism to remove the buffer after applying user-defined functions,
-#' since this task is specific to each process. In other `lidR` functions this task is performed
-#' specifically for each function. In `catalog_apply()` the user's function can return any output,
-#' thus users must take care of this task themselves (See section "Edge artifacts")
+#' \code{LAScatalog}. It is flexible and powerful but also complex. `catalog_map()` is a simplified version
+#' of `catalog_apply()` that suits for 90% of use cases.\cr\cr
+#' `catalog_sapply()` is a previous attempt to provide simplified version of `catalog_apply()`. Use
+#' `catalog_map()` instead.
 #'
 #' @param ctg A \link[=LAScatalog-class]{LAScatalog} object.
 #' @param FUN A user-defined function that respects a given template (see section function template)
@@ -49,17 +17,19 @@
 #' @section Edge artifacts:
 #'
 #' It is important to take precautions to avoid 'edge artifacts' when processing wall-to-wall
-#' tiles. If the points from neighboring tiles are not included during certain processes,
+#' tiles. If the points from neighbouring tiles are not included during certain processes,
 #' this could create 'edge artifacts' at the tile boundaries. For example, empty or incomplete
 #' pixels in a rasterization process, or dummy elevations in a ground interpolation. The LAScatalog
-#' processing engine provides internal tools to load buffered data 'on-the-fly'. However, there is
-#' no mechanism to remove the results computed in the buffered area since this task depends on the
-#' output of the user-defined function. The user must take care of this task (see examples) to prevent
-#' unexpected output with duplicated entries or conflict between values computed twice.
+#' processing engine provides internal tools to load buffered data 'on-the-fly'. `catalog_map()` takes
+#' care of removing automatically the results computed in the buffered area to avoid unexpected output
+#' with duplicated entries or conflict between values computed twice. It does that in predefined way
+#' that may not suit all cases. `catalog_apply()` does not remove the buffer and leave users free
+#' to handle this in a custom way. This is why `catalog_apply()` is more complex but gives more freedom
+#' to build new applications.
 #'
 #' @section Buffered data:
 #'
-#' The LAS objects read by the user function have a special attribute called 'buffer' that indicates,
+#' The LAS objects loaded in theses functions have a special attribute called 'buffer' that indicates,
 #' for each point, if it comes from a buffered area or not. Points from non-buffered areas have a
 #' 'buffer' value of 0, while points from buffered areas have a 'buffer' value of 1, 2, 3 or 4, where
 #' 1 is the bottom buffer and 2, 3 and 4 are the left, top and right buffers, respectively. This allows
@@ -67,39 +37,51 @@
 #'
 #' @section Function template:
 #'
-#' The parameter `FUN` expects a function with a first argument that will be supplied automatically
-#' by the `LAScatalog` processing engine. This first argument is a `LAScluster`. A `LAScluster`
-#' is an internal undocumented class but the user needs to know only three things about this class:
+#' The parameter `FUN` of `catalog_apply` expects a function with a first argument that will be
+#' supplied automatically by the `LAScatalog` processing engine. This first argument is a `LAScluster`.
+#' A `LAScluster` is an internal undocumented class but the user needs to know only three things about
+#' this class:
 #'
-#' - It represents a chunk of the catalog
+#' - It represents a chunk of the file collection
 #' - The function \link{readLAS} can be used with a `LAScluster`
-#' - The function \link[raster:extent]{extent} or \link[sp:bbox]{bbox} can be used with a `LAScluster`
-#' and it returns the bounding box of the cluster without the buffer. It can be used to clip the output
-#' and remove the buffered region (see examples).
+#' - The function \link[raster:extent]{extent} or \link[sp:bbox]{bbox} or \link[sf:st_bbox]{st_bbox}
+#' can be used with a `LAScluster` and they return the bounding box of the chunk without the buffer.
+#' It must be used to clip the output and remove the buffered region (see examples).
 #'
 #' A user-defined function must be templated like this:
 #'
 #' ```
-#' myfun <- function(cluster, ...) {
-#'    las <- readLAS(cluster)
+#' myfun <- function(chunk, ...) {
+#'    # Load the chunk + buffer
+#'    las <- readLAS(chunk)
 #'    if (is.empty(las)) return(NULL)
+#'
 #'    # do something
+#'    output <- do_something(las, ...)
+#'
 #'    # remove the buffer of the output
-#'    return(something)
+#'    bbox <- bbox(chunk)
+#'    output <- remove_buffer(output, bbox)
+#'    return(output)
 #' }
 #' ```
 #'
-#' The line `if(is.empty(las)) return(NULL)` is important because some clusters (chunks) may contain
+#' The line `if (is.empty(las)) return(NULL)` is important because some clusters (chunks) may contain
 #' 0 points (we can't know this before reading the file). In this case an empty point cloud with 0 points
 #' is returned by `readLAS()` and this may fail in subsequent code. Thus, exiting early from the user-defined
-#' function by returning `NULL` indicates to the internal engine that the cluster was empty.
+#' function by returning `NULL` indicates to the internal engine that the chunk was empty.
 #'
-#' From v3.0.0 if `autoread = TRUE` the following template is accepted because the engine takes care
-#' of the above mentionned steps:
+#' `catalog_map` is much simpler (but less versatile). It automatically takes care of reading the chunk
+#' and checks if the point cloud is empty. It also automatically crop the buffer. The way it crops the
+#' buffer suits for most cases but for some special cases it may be advised to handle this in a more
+#' specific way i.e. using `catalog_apply()`. For `catalog_map()` the first argument is a `LAS` and the
+#' template is:
 #'
 #' ```
-#' myfun <- function(las, bbox ...) {
+#' myfun <- function(las, ...) {
 #'    # do something
+#'    output <- do_something(las, ...)
+#'    return(output)
 #' }
 #' ```
 #'
@@ -107,21 +89,26 @@
 #' Users may have noticed that some lidR functions throw an error when the processing options are
 #' inappropriate. For example, some functions need a buffer and thus `buffer = 0` is forbidden.
 #' Users can add the same constraints to protect against inappropriate options. The `.options`
-#' argument is a `list` that allows users to tune the behavior of the processing engine.
+#' argument is a `list` that allows users to tune the behaviour of the processing engine.
 #'
-#' - `drop_null = FALSE` Not intended to be used by regular users. The engine does not remove
+#' - `drop_null = FALSE`: not intended to be used by regular users. The engine does not remove
 #' NULL outputs
-#' - `need_buffer = TRUE` the function complains if the buffer is 0.
+#' - `need_buffer = TRUE`: the function complains if the buffer is 0.
 #' - `need_output_file = TRUE` the function complains if no output file template is provided.
 #' - `raster_alignment = ...` the function checks the alignment of the chunks. This option is
 #' important if the output is a raster. See below for more details.
-#' - `automerge = TRUE` by defaut the engine returns a `list` with one item per chunk. If
+#' - `automerge = TRUE` by default the engine returns a `list` with one item per chunk. If
 #' `automerge = TRUE`, it tries to merge the outputs into a single object: a `Raster*`, a
-#' `Spatial*`, a `LAS*` similar to other functions of the package. This is a fail-safe
-#' option so in the worst case, if the merge fails, the `list` is returned.
+#' `Spatial*`, a `LAS*`, an `sf`, a ` stars` similarly to other functions of the package. This is a
+#'  fail-safe option so in the worst case, if the merging fails, the `list` is returned. This is
+#'  activated by `catalog_map` making it simpler.
 #' - `autoread = TRUE`. Introduced in v3.0.0 this option enables to get rid of the first steps of the
 #' function i.e `readLAS()` and `if (is.empty())`. In this case the function must take two
-#' objects as input, first a `LAS` object and second a `Extent` from `raster`.
+#' objects as input, first a `LAS` object and second a `bbox` from `sf`. This is
+#' activated by `catalog_map` making it simpler.
+#' - `autocrop = TRUE`. Introduced in v4.0.0 this option enables to get rid of the buffer crop steps
+#' of the function i.e `something <- remove_buffer(something, bbox)`. In this case the function must
+#' take one `LAS` object as input. This is activated by `catalog_map` making it simpler.
 #'
 #' When the function `FUN` returns a raster it is important to ensure that the chunks are aligned
 #' with the raster to avoid edge artifacts. Indeed, if the edge of a chunk does not correspond to the edge
@@ -139,106 +126,113 @@
 #' raster_alignment = list(res = 20, start = c(0,10))
 #' ```
 #'
-#' See also \link{grid_metrics} for more details.
-#'
-#' @section Supported processing options:
-#' Supported processing options for a \code{LAScatalog} (in bold). For more details see the
-#' \link[=LAScatalog-class]{LAScatalog engine documentation}:
-#'
-#' - *chunk_size*: How much data is loaded at once.
-#' - *chunk_buffer*: Load chunks with a buffer.
-#' - *chunk_alignment*: Align the chunks.
-#' - *progress*: Displays a progress estimate.
-#' - *output_files*: The user-defined function outputs will be written to files instead of being
-#' returned into R.
-#' - *laz_compression*: write `las` or `laz` files only if the user-defined function
-#' returns a `LAS`` object.
-#' - *select*: Select only the data of interest to save processing memory.
-#' - *filter*: Read only the points of interest.
-#'
 #' @examples
 #' # More examples might be avaible in the official lidR vignettes or
 #' # on the github book <https://jean-romain.github.io/lidRbook/>
 #'
 #' ## =========================================================================
 #' ## Example 1: detect all the tree tops over an entire catalog
-#' ## (this is basically a reproduction of the existing lidR function 'tree_detection')
+#' ## (this is basically a reproduction of the existing function 'locate_trees')
 #' ## =========================================================================
 #'
 #' # 1. Build the user-defined function that analyzes each chunk of the catalog.
 #' # The function's first argument is a LAScluster object. The other arguments can be freely
-#' # choosen by the user.
-#' my_tree_detection_method <- function(cluster, ws)
+#' # chosen by the users.
+#' my_tree_detection_method <- function(chunk, ws)
 #' {
-#'   # The cluster argument is a LAScluster object. The user does not need to know how it works.
+#'   # The chunk argument is a LAScluster object. The users do not need to know how it works.
 #'   # readLAS will load the region of interest (chunk) with a buffer around it, taking advantage of
 #'   # point cloud indexation if possible. The filter and select options are propagated automatically
-#'   las <- readLAS(cluster)
+#'   las <- readLAS(chunk)
 #'   if (is.empty(las)) return(NULL)
 #'
-#'   # Find the tree tops using a user-developed method (here simply a LMF).
-#'   ttops <- find_trees(las, lmf(ws))
+#'   # Find the tree tops using a user-developed method
+#'   # (here simply a LMF for the example).
+#'   ttops <- locate_trees(las, lmf(ws))
 #'
-#'   # ttops is a SpatialPointsDataFrame that contains the tree tops in our region of interest
+#'   # ttops is an sf object that contains the tree tops in our region of interest
 #'   # plus the trees tops in the buffered area. We need to remove the buffer otherwise we will get
 #'   # some trees more than once.
-#'   bbox  <- raster::extent(cluster)
-#'   ttops <- raster::crop(ttops, bbox)
-#'
+#'   bbox  <- st_bbox(chunk)
+#'   ttops <- sf::st_crop(ttops, bbox)
 #'   return(ttops)
 #' }
 #'
-#' # 2. Build a project (here, a single file catalog for the purposes of this dummmy example).
+#' # 2. Build a collection of file
+#' # (here, a single file LAScatalog for the purposes of this simple example).
 #' LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
-#' prj <- readLAScatalog(LASfile)
-#' plot(prj)
+#' ctg <- readLAScatalog(LASfile)
+#' plot(ctg)
 #'
 #' # 3. Set some processing options.
-#' # For this dummy example, the chunk size is 100 m and the buffer is 10 m
-#' opt_chunk_buffer(prj) <- 10
-#' opt_chunk_size(prj)   <- 100            # small because this is a dummy example.
-#' opt_chunk_alignment(prj) <- c(-50, -35) # Align such as it creates 2 chunks only.
-#' opt_select(prj)       <- "xyz"          # Read only the coordinates.
-#' opt_filter(prj)       <- "-keep_first"  # Read only first returns.
+#' # For this small single file example, the chunk size is 100 m + 10 m of buffer
+#' opt_chunk_buffer(ctg) <- 10
+#' opt_chunk_size(ctg)   <- 100            # Small because this is a dummy example.
+#' opt_chunk_alignment(ctg) <- c(-50, -35) # Align such as it creates 2 chunks only.
+#' opt_select(ctg)       <- "xyz"          # Read only the coordinates.
+#' opt_filter(ctg)       <- "-keep_first"  # Read only first returns.
 #'
 #' # 4. Apply a user-defined function to take advantage of the internal engine
 #' opt    <- list(need_buffer = TRUE,   # catalog_apply will throw an error if buffer = 0
 #'                automerge   = TRUE)   # catalog_apply will merge the outputs into a single object
-#' output <- catalog_apply(prj, my_tree_detection_method, ws = 5, .options = opt)
+#' output <- catalog_apply(ctg, my_tree_detection_method, ws = 5, .options = opt)
 #'
-#' spplot(output)
+#' plot(output)
 #'
-#' \dontrun{
+#' \donttest{
+#' ## =========================================================================
+#' ## Example 1: simplified. There is nothing that requires special data
+#' ## manipulation in the previous example. Everything can be handled automatically
+#' ##=========================================================================
+#'
+#' # 1. Build the user-defined function that analyzes a point cloud.
+#' my_tree_detection_method <- function(las, ws)
+#' {
+#'   # Find the tree tops using a user-developed method
+#'   # (here simply a LMF for the example).
+#'   ttops <- locate_trees(las, lmf(ws))
+#'   return(ttops)
+#' }
+#'
+#' # 2. Build a project
+#' LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
+#' ctg <- readLAScatalog(LASfile)
+#' plot(ctg)
+#'
+#' # 3. Set some processing options.
+#' # For this dummy example, the chunk size is 100 m and the buffer is 10 m
+#' opt_chunk_buffer(ctg) <- 10
+#' opt_chunk_size(ctg)   <- 100            # small because this is a dummy example.
+#' opt_chunk_alignment(ctg) <- c(-50, -35) # Align such as it creates 2 chunks only.
+#' opt_select(ctg)       <- "xyz"          # Read only the coordinates.
+#' opt_filter(ctg)       <- "-keep_first"  # Read only first returns.
+#'
+#' # 4. Apply a user-defined function to take advantage of the internal engine
+#' opt    <- list(need_buffer = TRUE)   # catalog_apply will throw an error if buffer = 0
+#' output <- catalog_map(ctg, my_tree_detection_method, ws = 5, .options = opt)
 #'
 #' ## ===================================================
 #' ## Example 2: compute a rumple index on surface points
 #' ## ===================================================
 #'
-#' rumple_index_surface = function(cluster, res)
+#' rumple_index_surface = function(las, res)
 #' {
-#'   las = readLAS(cluster)
-#'   if (is.empty(las)) return(NULL)
-#'
 #'   las    <- filter_surfacepoints(las, 1)
-#'   rumple <- grid_metrics(las, ~rumple_index(X,Y,Z), res)
-#'   bbox   <- raster::extent(cluster)
-#'   rumple <- raster::crop(rumple, bbox)
-#'
+#'   rumple <- pixel_metrics(las, ~rumple_index(X,Y,Z), res)
 #'   return(rumple)
 #' }
 #'
 #' LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
-#' prj <- readLAScatalog(LASfile)
+#' ctg <- readLAScatalog(LASfile)
 #'
-#' opt_chunk_buffer(prj) <- 1
-#' opt_chunk_size(prj)   <- 140     # small because this is a dummy example.
-#' opt_select(prj)       <- "xyz"   # read only the coordinates.
+#' opt_chunk_buffer(ctg) <- 1
+#' opt_chunk_size(ctg)   <- 140     # small because this is a dummy example.
+#' opt_select(ctg)       <- "xyz"   # read only the coordinates.
 #'
-#' opt     <- list(raster_alignment = 20, # catalog_apply will adjust the chunks if required
-#'                 automerge = TRUE)      # catalog_apply will merge the outputs into a single raster
-#' output  <- catalog_apply(prj, rumple_index_surface, res = 20, .options = opt)
+#' opt    <- list(raster_alignment = 20) # catalog_apply will adjust the chunks if required
+#' output <- catalog_map(ctg, rumple_index_surface, res = 20, .options = opt)
 #'
-#' plot(output, col = height.colors(50))
+#' plot(output, col = height.colors(25))
 #' }
 #' @export
 #' @md
@@ -259,7 +253,7 @@ catalog_apply <- function(ctg, FUN, ..., .options = NULL)
   realigment <- if (opt[["check_alignment"]]) realigment = opt[["raster_alignment"]] else FALSE
 
   # Produce the chunks
-  clusters <- catalog_makechunks(ctg, realigment)
+  clusters <- engine_chunks(ctg, realigment)
 
   # Disable the progress bar of the functions and ensure user options are restored
   oldstate <- getOption("lidR.progress")
@@ -267,7 +261,7 @@ catalog_apply <- function(ctg, FUN, ..., .options = NULL)
   on.exit(options(lidR.progress = oldstate), add = TRUE)
 
   # Process with the catalog processing engine
-  output <- cluster_apply(clusters, FUN, ctg@processing_options, ctg@output_options, opt[["globals"]], opt[["autoread"]], ...)
+  output <- engine_apply(clusters, FUN, ctg@processing_options, ctg@output_options, opt[["globals"]], opt[["autoread"]], opt[["autocrop"]], ...)
 
   # Filter NULLs and return
   if (isTRUE(opt[["drop_null"]]))
@@ -275,7 +269,7 @@ catalog_apply <- function(ctg, FUN, ..., .options = NULL)
 
   # Automerge
   if (!isFALSE(opt[["automerge"]]) && opt_merge(ctg))
-    output <- catalog_merge_results(ctg, output, opt[["automerge"]], as.character(substitute(FUN)))
+    output <- engine_merge(ctg, output, as.character(substitute(FUN)))
 
   return(output)
 }
@@ -287,7 +281,27 @@ catalog_sapply <- function(ctg, FUN, ..., .options = NULL)
   if (is.null(.options))
     .options <- list(automerge = TRUE)
   else
+  {
     .options$automerge <- TRUE
+  }
+
+  return(catalog_apply(ctg, FUN, ..., .options = .options))
+}
+
+#' @export
+#' @rdname catalog_apply
+catalog_map <- function(ctg, FUN, ..., .options = NULL)
+{
+  if (is.null(.options))
+    .options <- list(automerge = TRUE,
+                     autocrop  = TRUE,
+                     autoread  = TRUE)
+  else
+  {
+    .options$automerge <- TRUE
+    .options$autocrop  <- TRUE
+    .options$autoread  <- TRUE
+  }
 
   return(catalog_apply(ctg, FUN, ..., .options = .options))
 }
@@ -416,6 +430,22 @@ engine_parse_options = function(.option)
 
   output$autoread <- autoread
 
+  # autocrop
+
+  if (is.null(.option$autocrop))
+  {
+    autocrop <- FALSE
+  }
+  else
+  {
+    autocrop <- .option$autocrop
+
+    if (isTRUE(autocrop))
+      autocrop <- TRUE
+  }
+
+  output$autocrop <- autocrop
+
   # need buffer
 
   if (is.null(.option$need_buffer))
